
/* calendar, simple GTK2 calendar program, version 04 */
/* Arktis Desktop Project, 2021, GNU GPLv3 or any later */

#include <gtk/gtk.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <libintl.h>
#define _(X) gettext(X)
#define PACKAGE "puppy"

static GtkWidget *cwn;

static void caldx (gpointer calendar) {
	time_t secs;
	struct tm *tm;
	secs = time (NULL);
	tm = localtime (&secs);
	guint sday, smonth, syear, month, year;
	sday = tm->tm_mday;
	smonth = tm->tm_mon;
	syear  = 1900 + tm->tm_year;
	gtk_calendar_get_date (GTK_CALENDAR (calendar), &year, &month, NULL);
	if ( year == syear && month == smonth ) {
		gtk_calendar_select_day (GTK_CALENDAR (calendar), sday);
		gtk_calendar_mark_day (GTK_CALENDAR (calendar), sday);
	} else {
		gtk_calendar_clear_marks (GTK_CALENDAR (calendar));
		gtk_calendar_select_day (GTK_CALENDAR (calendar), 0);
	}
}

static void calck () {
	execlp ("/bin/sh", "sh", "-c", "defaultcalendar &", NULL);
}

static void calst () {
	execlp("/bin/sh", "sh", "-c", "set-time-for-puppy &", NULL);
}

static void calvt (GtkWidget *widget, GdkEventKey *event, gpointer gdata) {
	switch(event->keyval) {
		case GDK_Return:
		case GDK_Escape:
			gtk_main_quit();
			break;
		default:
			return;
	}
}

static void  calcw (GtkWidget *widget, GdkEventButton *event) {
	if (event->type == GDK_BUTTON_PRESS) {
		switch (event->button) {
			case 1:
				gtk_window_begin_move_drag (GTK_WINDOW (cwn),
					event->button, event->x_root, event->y_root, event->time);
			break;
		default:
			return;
		}
	}
}

int main( int argc, char *argv[] ) {
	GtkWidget *abtn, *aimg, *bbtn, *bimg, *calendar, *cbtn, *cimg, *datebox;
	GtkWidget *ebox, *fbox, *label, *vbox;

	textdomain(PACKAGE);

	gtk_init (&argc, &argv);

	cwn = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (cwn), _("Calendar"));
	gtk_widget_set_name (cwn, "calwindow");
	g_signal_connect (G_OBJECT (cwn), "destroy",
			G_CALLBACK (gtk_main_quit), NULL);
	gtk_window_set_decorated (GTK_WINDOW (cwn), FALSE);
	gtk_window_set_skip_pager_hint (GTK_WINDOW (cwn), TRUE);
	
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (cwn), TRUE);
	
	gtk_window_set_position (GTK_WINDOW (cwn), GTK_WIN_POS_MOUSE);
	fbox = gtk_frame_new (NULL);
	gtk_widget_set_name(fbox, "calborder");
	gtk_container_add (GTK_CONTAINER (cwn), fbox);

	vbox = gtk_vbox_new (FALSE, 2);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
	gtk_container_add (GTK_CONTAINER (fbox), vbox);
	datebox = gtk_hbox_new (FALSE, 4);

	ebox = gtk_event_box_new ();
	gtk_container_add (GTK_CONTAINER (datebox), ebox);
	g_signal_connect (G_OBJECT (ebox), "button_press_event",
			G_CALLBACK (calcw), NULL);
	gtk_event_box_set_above_child (GTK_EVENT_BOX (ebox), TRUE);
	gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);

	label = gtk_label_new ("");
	gtk_container_add (GTK_CONTAINER (ebox), label);

	aimg= gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
	abtn = gtk_button_new();
	gtk_widget_set_name(abtn, "cbtna");
	gtk_button_set_image (GTK_BUTTON (abtn), aimg);
	gtk_button_set_image_position (GTK_BUTTON (abtn), GTK_POS_TOP);
	gtk_button_set_relief (GTK_BUTTON (abtn), GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text (abtn,  _("Event"));
	g_signal_connect (abtn, "button-press-event", G_CALLBACK (calck), NULL);
	gtk_container_add (GTK_CONTAINER (datebox), abtn);

	bimg = gtk_image_new_from_stock (GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_BUTTON);
	bbtn = gtk_button_new();
	gtk_widget_set_name(bbtn, "cbtnb");
	gtk_button_set_image (GTK_BUTTON (bbtn), bimg);
	gtk_button_set_image_position (GTK_BUTTON (bbtn), GTK_POS_TOP);
	gtk_button_set_relief (GTK_BUTTON (bbtn), GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text (bbtn,  _("Set time"));
	g_signal_connect (bbtn, "button-press-event", G_CALLBACK (calst), NULL);
	gtk_container_add (GTK_CONTAINER (datebox), bbtn);

	cimg = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
	cbtn = gtk_button_new();
	gtk_widget_set_name(cbtn, "cbtnc");
	gtk_button_set_image (GTK_BUTTON (cbtn), cimg);
	gtk_button_set_image_position (GTK_BUTTON (cbtn), GTK_POS_TOP);
	gtk_button_set_relief (GTK_BUTTON (cbtn), GTK_RELIEF_NONE);
	gtk_widget_set_tooltip_text (cbtn,  _("Close"));
	g_signal_connect (cbtn, "button-press-event", G_CALLBACK (gtk_main_quit), NULL);
	g_signal_connect (cbtn, "key-press-event", G_CALLBACK (calvt), NULL);
	gtk_container_add (GTK_CONTAINER (datebox), cbtn);

	gtk_container_add (GTK_CONTAINER (vbox), datebox);

	calendar = gtk_calendar_new();
	gtk_widget_set_name(calendar, "cgrid");
	gtk_box_pack_start (GTK_BOX (vbox), calendar, FALSE, FALSE, 0);
	g_signal_connect (calendar, "month-changed",
						G_CALLBACK (caldx), G_OBJECT(calendar));
	g_signal_connect (calendar, "next-year", G_CALLBACK (caldx),
						G_OBJECT(calendar));
	g_signal_connect (calendar, "prev-year", G_CALLBACK (caldx),
						G_OBJECT(calendar));
	g_signal_connect (calendar, "show", G_CALLBACK (caldx), G_OBJECT(calendar));

	int i;
	for (i = 1; i < argc; ++i) {
		if(!strcmp(argv[i], "-w")) {
			GtkCalendarDisplayOptions flags;
			flags = GTK_CALENDAR_SHOW_HEADING |
						GTK_CALENDAR_SHOW_DAY_NAMES |
						GTK_CALENDAR_SHOW_WEEK_NUMBERS;
			gtk_calendar_set_display_options (GTK_CALENDAR (calendar), flags);
		}
		if (!strcmp(argv[i], "-t"))
			gtk_window_set_skip_taskbar_hint (GTK_WINDOW (cwn), FALSE);
		if (!strcmp(argv[i], "-s"))
			gtk_window_stick (GTK_WINDOW (cwn));
	}

	GDate date;
	gchar str[128];
	guint year, month, day;
	gtk_calendar_get_date (GTK_CALENDAR (calendar), &year, &month, &day);
	g_date_set_dmy (&date, day, month + 1, year);
	g_date_strftime (str, 127, " %A, %x    ", &date);
	gtk_label_set_text (GTK_LABEL (label), (gchar *) &str);

	gtk_widget_set_can_default (cbtn , TRUE);
	gtk_widget_set_can_focus (cbtn , TRUE);
	gtk_widget_grab_default (cbtn);
	gtk_widget_grab_focus (cbtn);

	gtk_widget_show_all (cwn);

	FILE *fa, *fc;
	char wa[] = "\0";
	fa = popen("which defaultcalendar", "r");
	fgets(wa, sizeof(wa), fa);
	if(wa[0]) {
		gtk_widget_set_visible (abtn, TRUE);
		g_signal_connect (calendar, "day-selected-double-click", 
						G_CALLBACK (calck), NULL);
	} else
		gtk_widget_set_visible (abtn, FALSE);
	pclose(fa);

	char wb[] = "\0";
	fc = popen("which set-time-for-puppy", "r");
	fgets(wb, sizeof(wb), fc);
	if(wb[0]) 
		gtk_widget_set_visible (bbtn, TRUE);
	else
		gtk_widget_set_visible (bbtn, FALSE);
	pclose(fc);

	gtk_main ();
	return 0;
}

